//+------------------------------------------------------------------+
//|                                                   SymbolInfo.mqh |
//|                      Copyright  2009, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Class CSymbolInfo.                                               |
//| Appointment: Class for access to symbol info.                    |
//+------------------------------------------------------------------+
class CSymbolInfo
  {
protected:
   string                      m_name;            // symbol name
   double                      m_point;           // symbol point
   double                      m_tick_value;      // symbol tick value
   double                      m_tick_size;       // symbol tick size
   double                      m_contract_size;   // symbol contract size
   double                      m_lots_min;        // symbol lots min
   double                      m_lots_max;        // symbol lots max
   double                      m_lots_step;       // symbol lots step
   double                      m_swap_long;       // symbol swap long
   double                      m_swap_short;      // symbol swap short
   int                         m_digits;          // symbol digits
   ENUM_SYMBOL_TRADE_EXECUTION m_trade_execution; // symbol trade execution
   ENUM_SYMBOL_CALC_MODE       m_trade_calcmode;  // symbol trade calcmode
   ENUM_SYMBOL_TRADE_MODE      m_trade_mode;      // symbol trade mode
   ENUM_SYMBOL_SWAP_MODE       m_swap_mode;       // symbol swap mode
   ENUM_DAY_OF_WEEK            m_swap3;           // symbol swap3

public:
                     CSymbolInfo();
   //--- methods of access to protected data
   string            Name()            { return(m_name);         }
   void              Name(string name) { m_name=name; Refresh(); }
   void              Refresh();
   //--- fast access methods to the integer symbol propertyes
   bool              Select();
   //--- volumes
   long              Volume();
   long              VolumeHigh();
   long              VolumeLow();
   long              VolumeBid();
   long              VolumeAsk();
   //--- miscellaneous
   datetime          Time();
   int               Spread();
   int               TicksBookDepth();
   //--- trade levels
   int               StopsLevel();
   int               FreezeLevel();
   //--- bid parameters
   double            Bid();
   double            BidHigh();
   double            BidLow();
   //--- ask parameters
   double            Ask();
   double            AskHigh();
   double            AskLow();
   //--- last parameters
   double            Last();
   double            LastHigh();
   double            LastLow();
   //--- terms of trade
   ENUM_SYMBOL_CALC_MODE  TradeCalcMode() { return(m_trade_calcmode); }
   string                 TradeCalcModeDescription();
   ENUM_SYMBOL_TRADE_MODE TradeMode()     { return(m_trade_mode); }
   string                 TradeModeDescription();
   //--- execution terms of trade
   ENUM_SYMBOL_TRADE_EXECUTION TradeExecution() { return(m_trade_execution); }
   string                      TradeExecutionDescription();
   //--- swap terms of trade
   ENUM_SYMBOL_SWAP_MODE SwapMode()          { return(m_swap_mode); }
   string                SwapModeDescription();
   ENUM_DAY_OF_WEEK      SwapRollover3days() { return(m_swap3); }
   string                SwapRollover3daysDescription();
   //--- tick parameters
   int               Digits()       { return(m_digits);        }
   double            Point()        { return(m_point);         }
   double            TickValue()    { return(m_tick_value);    }
   double            TickSize()     { return(m_tick_size);     }
   //--- lots parameters
   double            ContractSize() { return(m_contract_size); }
   double            LotsMin()      { return(m_lots_min);      }
   double            LotsMax()      { return(m_lots_max);      }
   double            LotsStep()     { return(m_lots_step);     }
   //--- swaps
   double            SwapLong()     { return(m_swap_long);     }
   double            SwapShort()    { return(m_swap_short);    }
   //--- fast access methods to the string symbol propertyes
   string            CurrencyBase();
   string            CurrencyProfit();
   string            CurrencyMargin();
   string            Bank();
   string            Description();
   string            Path();
  };
//+------------------------------------------------------------------+
//| Constructor CSymbolInfo.                                         |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CSymbolInfo::CSymbolInfo()
  {
   m_name           ="";
   m_point          =0.0;
   m_tick_value     =0.0;
   m_tick_size      =0.0;
   m_contract_size  =0.0;
   m_lots_min       =0.0;
   m_lots_max       =0.0;
   m_lots_step      =0.0;
   m_swap_long      =0.0;
   m_swap_short     =0.0;
   m_digits         =0;
   m_trade_execution=0;
   m_trade_calcmode =0;
   m_trade_mode     =0;
   m_swap_mode      =0;
   m_swap3          =0;
  }
//+------------------------------------------------------------------+
//| Refresh cached data                                              |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CSymbolInfo::Refresh()
  {
   m_point          =SymbolInfoDouble(m_name,SYMBOL_POINT);
   m_tick_value     =SymbolInfoDouble(m_name,SYMBOL_TRADE_TICK_VALUE);
   m_tick_size      =SymbolInfoDouble(m_name,SYMBOL_TRADE_TICK_SIZE);
   m_contract_size  =SymbolInfoDouble(m_name,SYMBOL_TRADE_CONTRACT_SIZE);
   m_lots_min       =SymbolInfoDouble(m_name,SYMBOL_VOLUME_MIN);
   m_lots_max       =SymbolInfoDouble(m_name,SYMBOL_VOLUME_MAX);
   m_lots_step      =SymbolInfoDouble(m_name,SYMBOL_VOLUME_STEP);
   m_swap_long      =SymbolInfoDouble(m_name,SYMBOL_SWAP_LONG);
   m_swap_short     =SymbolInfoDouble(m_name,SYMBOL_SWAP_SHORT);
   m_digits         =(int)SymbolInfoInteger(m_name,SYMBOL_DIGITS);
   m_trade_execution=(ENUM_SYMBOL_TRADE_EXECUTION)SymbolInfoInteger(m_name,SYMBOL_TRADE_EXEMODE);
   m_trade_calcmode =(ENUM_SYMBOL_CALC_MODE)SymbolInfoInteger(m_name,SYMBOL_TRADE_CALC_MODE);
   m_trade_mode     =(ENUM_SYMBOL_TRADE_MODE)SymbolInfoInteger(m_name,SYMBOL_TRADE_MODE);
   m_swap_mode      =(ENUM_SYMBOL_SWAP_MODE)SymbolInfoInteger(m_name,SYMBOL_SWAP_MODE);
   m_swap3          =(ENUM_DAY_OF_WEEK)SymbolInfoInteger(m_name,SYMBOL_SWAP_ROLLOVER3DAYS);
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_SELECT".                          |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_SELECT".                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSymbolInfo::Select()
  {
   return((bool)SymbolInfoInteger(m_name,SYMBOL_SELECT));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_VOLUME".                          |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_VOLUME".                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
long CSymbolInfo::Volume()
  {
   return(SymbolInfoInteger(m_name,SYMBOL_VOLUME));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_VOLUMEHIGH".                      |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_VOLUMEHIGH".                  |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
long CSymbolInfo::VolumeHigh()
  {
   return(SymbolInfoInteger(m_name,SYMBOL_VOLUMEHIGH));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_VOLUMELOW".                       |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_VOLUMELOW".                   |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
long CSymbolInfo::VolumeLow()
  {
   return(SymbolInfoInteger(m_name,SYMBOL_VOLUMELOW));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_VOLUMEBID".                       |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_VOLUMEBID".                   |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
long CSymbolInfo::VolumeBid()
  {
   return(SymbolInfoInteger(m_name,SYMBOL_VOLUMEBID));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_VOLUMEASK".                       |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_VOLUMEASK".                   |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
long CSymbolInfo::VolumeAsk()
  {
   return(SymbolInfoInteger(m_name,SYMBOL_VOLUMEASK));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_TIME".                            |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_TIME".                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
datetime CSymbolInfo::Time()
  {
   return((datetime)SymbolInfoInteger(m_name,SYMBOL_TIME));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_SPREAD".                          |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_SPREAD".                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CSymbolInfo::Spread()
  {
   return((int)SymbolInfoInteger(m_name,SYMBOL_SPREAD));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_TICKS_BOOKDEPTH".                 |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_TICKS_BOOKDEPTH".             |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CSymbolInfo::TicksBookDepth()
  {
   return((int)SymbolInfoInteger(m_name,SYMBOL_TICKS_BOOKDEPTH));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_TRADE_STOPS_LEVEL".               |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_TRADE_STOPS_LEVEL".           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CSymbolInfo::StopsLevel()
  {
   return((int)SymbolInfoInteger(m_name,SYMBOL_TRADE_STOPS_LEVEL));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_TRADE_FREEZE_LEVEL".              |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_TRADE_FREEZE_LEVEL".          |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CSymbolInfo::FreezeLevel()
  {
   return((int)SymbolInfoInteger(m_name,SYMBOL_TRADE_FREEZE_LEVEL));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_BID".                             |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_BID".                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CSymbolInfo::Bid()
  {
   return(SymbolInfoDouble(m_name,SYMBOL_BID));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_BIDHIGH".                         |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_BIDHIGH".                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CSymbolInfo::BidHigh()
  {
   return(SymbolInfoDouble(m_name,SYMBOL_BIDHIGH));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_BIDLOW".                          |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_BIDLOW".                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CSymbolInfo::BidLow()
  {
   return(SymbolInfoDouble(m_name,SYMBOL_BIDLOW));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_ASK".                             |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_ASK".                         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CSymbolInfo::Ask()
  {
   return(SymbolInfoDouble(m_name,SYMBOL_ASK));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_ASKHIGH".                         |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_ASKHIGH".                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CSymbolInfo::AskHigh()
  {
   return(SymbolInfoDouble(m_name,SYMBOL_ASKHIGH));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_ASKLOW".                          |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_ASKLOW".                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CSymbolInfo::AskLow()
  {
   return(SymbolInfoDouble(m_name,SYMBOL_ASKLOW));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_LAST".                            |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_LAST".                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CSymbolInfo::Last()
  {
   return(SymbolInfoDouble(m_name,SYMBOL_LAST));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_LASTHIGH".                        |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_LASTHIGH".                    |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CSymbolInfo::LastHigh()
  {
   return(SymbolInfoDouble(m_name,SYMBOL_LASTHIGH));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_LASTLOW".                         |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_LASTLOW".                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
double CSymbolInfo::LastLow()
  {
   return(SymbolInfoDouble(m_name,SYMBOL_LASTLOW));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_TRADE_CALC_MODE" as string.       |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_TRADE_CALC_MODE" as string.   |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CSymbolInfo::TradeCalcModeDescription()
  {
   string str;
//---
   switch(m_trade_calcmode)
     {
      case SYMBOL_CALC_MODE_FOREX:
         str="Calculation of profit and margin for Forex";
         break;
      case SYMBOL_CALC_MODE_CFD:
         str="Calculation of collateral and earnings for CFD";
         break;
      case SYMBOL_CALC_MODE_FUTURES:
         str="Calculation of collateral and profits for futures";
         break;
      case SYMBOL_CALC_MODE_CFDINDEX:
         str="Calculation of collateral and earnings for CFD on indices";
         break;
      case SYMBOL_CALC_MODE_CFDLEVERAGE:
         str="Calculation of collateral and earnings for the CFD when trading with leverage";
         break;
      default:
         str="Unknown calculation mode";
     }
//---
   return(str);
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_TRADE_MODE" as string.            |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_TRADE_MODE" as string.        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CSymbolInfo::TradeModeDescription()
  {
   string str;
//---
   switch(m_trade_mode)
     {
      case SYMBOL_TRADE_MODE_DISABLED:
         str="Trading symbol on the disabled";
         break;
      case SYMBOL_TRADE_MODE_LONGONLY:
         str="Allowed only long positions";
         break;
      case SYMBOL_TRADE_MODE_SHORTONLY:
         str="Allowed only short positions";
         break;
      case SYMBOL_TRADE_MODE_CLOSEONLY:
         str="Allowed only close positions";
         break;
      case SYMBOL_TRADE_MODE_FULL:
         str="No restrictions on trading";
         break;
      default:
         str="Unknown trade mode";
     }
//---
   return(str);
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_TRADE_EXEMODE" as string.         |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_TRADE_EXEMODE" as string.     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CSymbolInfo::TradeExecutionDescription()
  {
   string str;
//---
   switch(m_trade_execution)
     {
      case SYMBOL_TRADE_EXECUTION_REQUEST:
         str="Trading on request";
         break;
      case SYMBOL_TRADE_EXECUTION_INSTANT:
         str="Trading on live streaming prices";
         break;
      case SYMBOL_TRADE_EXECUTION_MARKET:
         str="Execution of orders on the market";
         break;
      default:
         str="Unknown trade execution";
     }
//---
   return(str);
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_SWAP_MODE" as string.             |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_SWAP_MODE" as string.         |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CSymbolInfo::SwapModeDescription()
  {
   string str;
//---
   switch(m_swap_mode)
     {
      case SYMBOL_SWAP_MODE_DISABLED:
         str="No swaps";
         break;
      case SYMBOL_SWAP_MODE_BY_POINTS:
         str="Swaps are calculated in points";
         break;
      case SYMBOL_SWAP_MODE_BY_MONEY:
         str="Swaps are calculated in of deposit currency";
         break;
      case SYMBOL_SWAP_MODE_BY_INTEREST:
         str="Swaps are calculated at an annual percentage";
         break;
      case SYMBOL_SWAP_MODE_BY_MARGIN_CURRENCY:
         str="Swaps are calculated in  of margin currency";
         break;
      default:
         str="Unknown swap mode";
     }
//---
   return(str);
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_SWAP_ROLLOVER3DAYS" as string.    |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_SWAP_ROLLOVER3DAYS" as string.|
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CSymbolInfo::SwapRollover3daysDescription()
  {
   string str;
//---
   switch(m_swap3)
     {
      case SUNDAY:
         str="Sunday";
         break;
      case MONDAY:
         str="Monday";
         break;
      case TUESDAY:
         str="Tuesday";
         break;
      case WEDNESDAY:
         str="Wednesday";
         break;
      case THURSDAY:
         str="Thursday";
         break;
      case FRIDAY:
         str="Friday";
         break;
      case SATURDAY:
         str="Saturday";
         break;
      default:
         str="Unknown";
     }
//---
   return(str);
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_CURRENCY_BASE".                   |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_CURRENCY_BASE".               |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CSymbolInfo::CurrencyBase()
  {
   return(SymbolInfoString(m_name,SYMBOL_CURRENCY_BASE));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_CURRENCY_PROFIT".                 |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_CURRENCY_PROFIT".             |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CSymbolInfo::CurrencyProfit()
  {
   return(SymbolInfoString(m_name,SYMBOL_CURRENCY_PROFIT));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_CURRENCY_MARGIN".                 |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_CURRENCY_MARGIN".             |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CSymbolInfo::CurrencyMargin()
  {
   return(SymbolInfoString(m_name,SYMBOL_CURRENCY_MARGIN));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_BANK".                            |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_BANK".                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CSymbolInfo::Bank()
  {
   return(SymbolInfoString(m_name,SYMBOL_BANK));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_DESCRIPTION".                     |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_DESCRIPTION".                 |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CSymbolInfo::Description()
  {
   return(SymbolInfoString(m_name,SYMBOL_DESCRIPTION));
  }
//+------------------------------------------------------------------+
//| Get the property value "SYMBOL_PATH".                            |
//| INPUT:  no.                                                      |
//| OUTPUT: the property value "SYMBOL_PATH".                        |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
string CSymbolInfo::Path()
  {
   return(SymbolInfoString(m_name,SYMBOL_PATH));
  }
//+------------------------------------------------------------------+
